# --- CONFIGURATION ---
$apiKey = 'YOUR_API_KEY'
$lat    = 37.7749
$lon    = -122.4194
$units  = 'imperial'  # Options: 'imperial', 'metric', 'standard'
$outputFolder = "C:\WeatherOutput"

# --- CREATE OUTPUT FOLDER ---
if (-not (Test-Path -LiteralPath $outputFolder)) {
    New-Item -ItemType Directory -Path $outputFolder -Force | Out-Null
}

# --- FETCH WEATHER DATA ---
$url = "https://api.openweathermap.org/data/2.5/weather?lat=$lat&lon=$lon&appid=$apiKey&units=$units"
try {
    $response = Invoke-RestMethod -Uri $url -Method Get -ErrorAction Stop
} catch {
    Write-Error "Failed to get weather data: $_"
    return
}

# --- UNIT LABELS ---
switch ($units) {
    'imperial' { $temp_unit = 'F'; $speed_unit = 'mph'; $pressure_unit = 'hPa'; $precip_unit = 'in' }
    'metric'   { $temp_unit = 'C'; $speed_unit = 'm/s'; $pressure_unit = 'hPa'; $precip_unit = 'mm' }
    default    { $temp_unit = 'K'; $speed_unit = 'm/s'; $pressure_unit = 'hPa'; $precip_unit = 'mm' }
}

# --- HELPER: Write UTF-8 w/o BOM ---
function Write-TextFile {
    param (
        [string]$Path,
        [string]$Value
    )
    $utf8 = [System.Text.UTF8Encoding]::new($false)
    [System.IO.File]::WriteAllText($Path, $Value, $utf8)
}

# --- HELPER: Compass Label from Degrees ---
function Get-CompassDirection {
    param ([double]$deg)
    $dirs = @("N","NNE","NE","ENE","E","ESE","SE","SSE","S","SSW","SW","WSW","W","WNW","NW","NNW")
    return $dirs[[math]::Round($deg / 22.5) % 16]
}

# --- EXTRACT & FORMAT VALUES ---
$station_name     = $response.name
$weather_desc     = $response.weather[0].description
$feels_like       = "$($response.main.feels_like)$temp_unit"
$temp_min         = "$($response.main.temp_min)$temp_unit"
$temp_max         = "$($response.main.temp_max)$temp_unit"
$wind_speed       = "$($response.wind.speed) $speed_unit"
$visibility       = "$($response.visibility) m"
$pressure         = "$($response.main.pressure) $pressure_unit"
$humidity         = "$($response.main.humidity)%"

# --- Rain and snow (1h and 3h) ---
$rain_1h = if ($response.PSObject.Properties.Name -contains 'rain' -and $response.rain.PSObject.Properties.Name -contains '1h') {
    "$($response.rain.'1h') $precip_unit"
} else {
    "N/A"
}
$rain_3h = if ($response.PSObject.Properties.Name -contains 'rain' -and $response.rain.PSObject.Properties.Name -contains '3h') {
    "$($response.rain.'3h') $precip_unit"
} else {
    "N/A"
}
$snow_1h = if ($response.PSObject.Properties.Name -contains 'snow' -and $response.snow.PSObject.Properties.Name -contains '1h') {
    "$($response.snow.'1h') $precip_unit"
} else {
    "N/A"
}
$snow_3h = if ($response.PSObject.Properties.Name -contains 'snow' -and $response.snow.PSObject.Properties.Name -contains '3h') {
    "$($response.snow.'3h') $precip_unit"
} else {
    "N/A"
}

$sea_level = if ($response.main.PSObject.Properties.Name -contains 'sea_level') {
    "$($response.main.sea_level) $pressure_unit"
} else {
    "N/A"
}

$last_update = ([DateTimeOffset]::FromUnixTimeSeconds($response.dt)).ToString("yyyy-MM-dd HH:mm:ss zzz")

if ($response.wind.PSObject.Properties.Name -contains 'deg') {
    $wind_dir_deg = $response.wind.deg
    $wind_dir_compass = Get-CompassDirection $wind_dir_deg
} else {
    $wind_dir_deg = "N/A"
    $wind_dir_compass = "N/A"
}

$wind_gust = if ($response.wind.PSObject.Properties.Name -contains 'gust') {
    "$($response.wind.gust) $speed_unit"
} else {
    "N/A"
}

# --- DISPLAY TO CONSOLE ---
Write-Host "Station name:          $station_name"
Write-Host "Weather description:   $weather_desc"
Write-Host "Feels like temp:       $feels_like"
Write-Host "Min temperature:       $temp_min"
Write-Host "Max temperature:       $temp_max"
Write-Host "Rain (1h):             $rain_1h"
Write-Host "Rain (3h):             $rain_3h"
Write-Host "Snow (1h):             $snow_1h"
Write-Host "Snow (3h):             $snow_3h"
Write-Host "Wind speed:            $wind_speed"
Write-Host "Wind direction (deg):  $wind_dir_deg"
Write-Host "Wind direction (comp): $wind_dir_compass"
Write-Host "Wind gust:             $wind_gust"
Write-Host "Visibility:            $visibility"
Write-Host "Barometric pressure:   $pressure"
Write-Host "Humidity:              $humidity"
Write-Host "Sea level:             $sea_level"
Write-Host "Last updated:          $last_update"

# --- WRITE TO FILES ---
Write-TextFile -Path "$outputFolder\station_name.txt"     -Value $station_name
Write-TextFile -Path "$outputFolder\weather_desc.txt"     -Value $weather_desc
Write-TextFile -Path "$outputFolder\feels_like.txt"       -Value $feels_like
Write-TextFile -Path "$outputFolder\temp_min.txt"         -Value $temp_min
Write-TextFile -Path "$outputFolder\temp_max.txt"         -Value $temp_max
Write-TextFile -Path "$outputFolder\rain_1h.txt"          -Value $rain_1h
Write-TextFile -Path "$outputFolder\rain_3h.txt"          -Value $rain_3h
Write-TextFile -Path "$outputFolder\snow_1h.txt"          -Value $snow_1h
Write-TextFile -Path "$outputFolder\snow_3h.txt"          -Value $snow_3h
Write-TextFile -Path "$outputFolder\wind_speed.txt"       -Value $wind_speed
Write-TextFile -Path "$outputFolder\wind_dir_deg.txt"     -Value "$wind_dir_deg"
Write-TextFile -Path "$outputFolder\wind_dir_compass.txt" -Value $wind_dir_compass
Write-TextFile -Path "$outputFolder\wind_gust.txt"        -Value $wind_gust
Write-TextFile -Path "$outputFolder\visibility.txt"       -Value $visibility
Write-TextFile -Path "$outputFolder\pressure.txt"         -Value $pressure
Write-TextFile -Path "$outputFolder\humidity.txt"         -Value $humidity
Write-TextFile -Path "$outputFolder\sea_level.txt"        -Value $sea_level
Write-TextFile -Path "$outputFolder\last_update.txt"      -Value $last_update
